import xml.etree.ElementTree as ET
from os import path

#input path
path_input ={ 'gait':'C:\OpenSimWorkspace\Models\Gait2392_Simbody\gait2392_simbody.osim', 
'thoracolumbar':'C:\OpenSimWorkspace\Models\Thoracolumbar_OSIM_V1\Thoracolumbar_Spine_With_RibCage.osim'}

#input: gait or thoracolumbar
model= 'gait'

tree = ET.parse(path_input[model])
root = tree.getroot()

#functions to help making body_structure
def getbodies():
    geometry_list_helper=[]
    for body in root.findall('.//Body'):
            geometry_list_helper.append(body.get('name'))
    return geometry_list_helper

#function that prints body name and corresponding geometry files 
def geometryfiles ():
    for body in root.findall('.//Body'):
        print '_______________________'
        print body.get('name')
        for geometry_file in body.findall('.//geometry_file'):
            geometry_file = geometry_file.text
            if geometry_file is not None:
                print geometry_file

#input: treestructure of bodies Thoracolumbar
ground=['ground', 'sacrum', 'pelvis']
lumbar=['lumbar3', 'lumbar4', 'lumbar3', 'lumbar2', 'lumbar1', 'lumbar5']
thoracic=['thoracic12', 'thoracic11', 'thoracic10', 'thoracic9', 'thoracic8', 'thoracic7', 'thoracic6', 'thoracic5', 'thoracic4', 'thoracic3', 'thoracic2', 'thoracic1']
rib=['sternum','rib12_R', 'rib11_R', 'rib10_R', 'rib9_R', 'rib8_R', 'rib7_R', 'rib6_R', 'rib5_R', 'rib4_R', 'rib3_R', 'rib2_R', 'rib1_R', 'rib12_L', 'rib11_L', 'rib10_L', 'rib9_L', 'rib8_L', 'rib7_L', 'rib6_L', 'rib5_L', 'rib4_L', 'rib3_L', 'rib2_L', 'rib1_L',]
abd=[ 'Abdomen','Abd_L_L1', 'Abd_L_L2', 'Abd_L_L3', 'Abd_L_L4', 'Abd_L_L5', 'Abd_R_L1', 'Abd_R_L2', 'Abd_R_L3', 'Abd_R_L4', 'Abd_R_L5']
arm=['clavicle_R', 'scapula_R', 'humerus_R', 'ulna_R', 'radius_R','clavicle_L', 'scapula_L', 'humerus_L', 'ulna_L', 'radius_L']
hand=['hand_R', 'hand_L']
headneck=['head_neck']

body_structure_thoracolumbar={'ground':ground, 'lumbar':lumbar,'thoracic':thoracic, 'rib':rib, 'abd':abd, 'arm':arm,'hand':hand,'headneck':headneck }

#input: treestructure of bodies gait2392
ground=['pelvis']
upper_body=['torso']
feet=['talus_r', 'calcn_r', 'toes_r', 'talus_l', 'calcn_l', 'toes_l']
legs=['femur_r', 'tibia_r', 'femur_l', 'tibia_l']

body_structure_gait2392={'ground':ground, 'upper_body':upper_body,'feet':feet, 'legs':legs }

body_structure={'gait':body_structure_gait2392, 'thoracolumbar':body_structure_thoracolumbar }
body_structure= body_structure[model]

#function for writing sub-geometry-class
def write_sub_geometry_class(structure, part, root):

    def write_class(class_name, cont):
        with open(path.relpath(class_name + '.aml'), 'a') as f:
            f.write(cont)
            print 'Class: "' + part + '" written to AML file.'
           
    sub_geometry_class=" "
    reference_object_list=" "
    for body in root.findall('.//Body'):
        if body.get('name') in structure[part]:
            reference_object_list +='' + body.attrib['name'] + '-reference-joint (default nil)\n'
            for DisplayGeometry in body.findall('.//DisplayGeometry'): 
                for geometry_file in DisplayGeometry.findall('.//geometry_file'):
                    geometry_file = geometry_file.text
                    if geometry_file is not None:
                        geometry_file= geometry_file.replace(".vtp","",1)
                        for  transform in  DisplayGeometry.findall('.//transform'):
                            rx, ry, rz, x, y, z =transform.text.split()
                            sub_geometry_class += "(" + geometry_file + " :class ' " + geometry_file + "-web-surface \n" + \
                            'orientation (list (rotate (radians-to-degrees ' + rx + ' ) :x-axis)(rotate (radians-to-degrees ' + ry + ' ) :y-axis)(rotate (radians-to-degrees ' + rz +' ) :z-axis)(translate (list '+ x +' '+ y +' '+ z +'))) \n' + \
                            'reference-coordinate-system  ^^' + body.attrib['name'] + '-reference-joint \n' +\
                            ')\n'
                              
    class_definition = '(define-class '+ part +'-sub-geometry-class \n' + \
                    ':inherit-from (\n' + \
                    'object\n' + \
                    ')\n' + \
                    ':properties (\n' + \
                    reference_object_list +\
                    ')\n' + \
                    ':subobjects (\n' + \
                    sub_geometry_class + \
                    ')\n' + \
                    ')\n'

    aml_code = class_definition
   
    write_class('sub-geometry_class_gait' , aml_code) #aml_code is the string created from functions defined to fit aml classes

#function for writing body-class
def write_body_class(structure, root):

    def write_class(class_name, cont):
        with open(path.relpath(class_name + '.aml'), 'a') as f:
            f.write(cont)
            print 'Class: body-class written to AML file.'

    class_definition = '(define-class body-class \n' + \
                    ':inherit-from (\n' + \
                    'object\n' + \
                    ')\n' + \
                    ':properties (\n' + \
                    ')\n' + \
                    ':subobjects (\n' +\
                    "(joints: class 'joint-class \n" +\
                    ')\n'
                  
    
    aml_code = class_definition

    write_class('body_class_gait', aml_code)
    
    geometry_list = " "           
    for part in structure:
        geometry_list = "(" + part + " :class '" + part +"-sub-geometry-class \n"
        reference_object_list = " "
        for body in root.findall('.//Body'):
            if body.get('name') in structure[part]:
                 reference_object_list +='' + body.attrib['name'] + '-reference-joint (the ' + body.attrib['name'] + '-joint (:from ^^joints))\n'
                                 
               
                                                  
        class_definition = geometry_list +\
                           reference_object_list +\
                            ')\n'
        
    
        aml_code = class_definition

        write_class('body_class_gait', aml_code)


#write to file. Remeber to comment out if you dont want to keep writing to file 
write_body_class(body_structure, root)

#for body_part in body_structure:
    #write_sub_geometry_class(body_structure, body_part, root) 
